<?php
/**
*
* @package phpBB3 SEO Sitemap
* @version $Id: functions_sitemap.php 8987 2014-01-18 07:38:32Z Shredder $
* @copyright (c) 2014 www.phpbb-work.ru
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Generate sitemap
*/
function generate_sitemap()
{
	global $auth, $config, $db, $user, $f_xml, $phpbb_seo, $urls, $phpbb_root_path, $phpEx;

	$excl = ($config['sitemap_seo_excluded']) ? explode(',', $config['sitemap_seo_excluded']) : array();
	$limit = $config['sitemap_seo_url_limit'];
	$seo = (isset($phpbb_seo)) ? $phpbb_seo->seo_opt['url_rewrite'] : 0;
	$path = generate_board_url();

	$user->setup('mods/info_acp_seo_sitemap');

	$f_beg = '<?xml version="1.0" encoding="UTF-8"?>'."\r\n";
	$f_beg .= '<?xml-stylesheet type="text/xsl" href="' . $path . '/styles/sitemap.xsl"?>'."\r\n";
	$f_beg .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'."\r\n";
	$f_end = '</urlset>';

	// Handle Prime trash bin and Soft Delete mods
	$soft = $soft_t = $soft_p = '';

	if (isset($config['topic_delete_mode']) || $auth->acl_get('m_harddelete') !== false)
	{
		$soft = 'AND topic_deleted_time = 0';
		$soft_p = 'AND p.post_deleted_time = 0';
		$soft_t = 'AND t.topic_deleted_time = 0';
	}

	// Select basic info
	$f_ids = array(0);

	$sql = 'SELECT f.forum_id, f.forum_type, f.forum_name, f.forum_topics_per_page, COUNT(t.topic_id) AS forum_topics
		FROM ' . FORUMS_TABLE . ' f, ' . TOPICS_TABLE . ' t
		WHERE t.topic_approved = 1
			' . $soft_t . '
			AND t.forum_id = f.forum_id
		GROUP BY f.forum_id';
	$result = $db->sql_query($sql);

	while ($row = $db->sql_fetchrow($result))
	{
		if (!in_array($row['forum_id'], $excl) && $auth->acl_get('f_list', $row['forum_id']) && $row['forum_type'] == FORUM_POST)
		{
			$f_ids[] = $row['forum_id'];
			$forums[$row['forum_id']] = $row;
		}
	}
	$db->sql_freeresult($result);

	// Generate forums list
	$index = 1;

	$sql = 'SELECT forum_id, topic_id, topic_type, topic_last_post_time
		FROM ' . TOPICS_TABLE . '
		WHERE topic_approved = 1
			AND ' . $db->sql_in_set('forum_id', $f_ids) . '
			' . $soft . '
		ORDER BY forum_id,
		CASE
			WHEN topic_type = ' . POST_ANNOUNCE . ' THEN 1
			' . ((defined('POST_ARTICLE')) ? 'WHEN topic_type = ' . POST_ARTICLE . ' THEN 2' : '') . '
			WHEN topic_type = ' . POST_STICKY . ' THEN 3
			ELSE 4
		END, topic_last_post_time DESC';
	$result = $db->sql_query($sql);

	$glob = $ann = $times = array();
	$page = $topic = $all_f = 0;

	while ($data = $db->sql_fetchrow($result))
	{
		$t_type = (int) $data['topic_type'];

		if ($t_type == POST_GLOBAL)
		{
			$glob[] = $data['topic_last_post_time'];
			continue;
		}
		else if ($t_type == POST_ANNOUNCE)
		{
			++$all_f;
			$ann[] = $data['topic_last_post_time'];
		}
		else
		{
			++$topic;
			++$all_f;
			$times[] = $data['topic_last_post_time'];
		}

		$per = ($forums[$data['forum_id']]['forum_topics_per_page']) ? $forums[$data['forum_id']]['forum_topics_per_page'] : $config['topics_per_page'];

		if ($topic == $per || $forums[$data['forum_id']]['forum_topics'] == $all_f)
		{
			$f_id = (int) $data['forum_id'];

			if ($seo)
			{
				if ($page == 0)
				{
					$phpbb_seo->set_url($forums[$f_id]['forum_name'], $f_id, $phpbb_seo->seo_static['forum']);
					$f_url = append_sid("{$phpbb_root_path}viewforum.$phpEx", 'f=' . $f_id);
				}
				else
				{
					$f_url = append_sid("{$phpbb_root_path}viewforum.$phpEx", 'f=' . $f_id . "&amp;start=" . ($page * $per));
				}

				$f_url = $phpbb_seo->drop_sid($f_url);
			}
			else
			{
				if ($page == 0)
				{
					$f_url = $path . "/viewforum.$phpEx?f=$f_id";
				}
				else
				{
					$f_url = $path . "/viewforum.$phpEx?f=$f_id&amp;start=" . ($page * $per);
				}
			}

			if ($urls && ($urls % ($limit * $index)) == 0)
			{
				$f_xml = $f_beg . $f_xml . $f_end;

				if (!($w = @fopen('sitemap/'.$index.'.xml', 'w')))
				{
					trigger_error(sprintf($user->lang['SEOMAP_CANT_WRITE'], $path . '/sitemap/'));
				}

				fwrite($w, $f_xml);
				fclose($w);

				$f_xml = '';
				++$index;
			}

			$time = gmdate('Y-m-d\TH:i:s+00:00', (int) max(array_merge($glob, $ann, $times)));

			add_to_record($f_url, $time, $config['sitemap_seo_freq_f'], $config['sitemap_seo_prior_f']);

			++$page;
			$times = array();
			$topic = 0;

			if ($forums[$f_id]['forum_topics'] == $all_f)
			{
				$page = $all_f = 0;
				$ann = array();
			}
		}
	}
	$db->sql_freeresult($result);

	// Generate topics list
	$seo_add = '';
	if ($seo)
	{
		$seo_add =  ', t.topic_url';
	}

	$sql = 'SELECT t.topic_id' . $seo_add . ', t.topic_type, t.topic_title, t.topic_last_post_id, t.forum_id, p.post_id, p.post_modified
		FROM ' . TOPICS_TABLE . ' t, ' . POSTS_TABLE . ' p
		WHERE t.topic_approved = 1 AND p.post_approved = 1 AND t.topic_status <> ' . ITEM_MOVED . '
			AND ' . $db->sql_in_set('t.forum_id', $f_ids) . '
			' . $soft_p . '
			' . $soft_t . '
			AND t.topic_id = p.topic_id
		ORDER BY t.topic_id, p.post_time';
	$result = $db->sql_query($sql);

	$times = array();
	$per = $config['posts_per_page'];
	$page = $post = 0;

	while ($data = $db->sql_fetchrow($result))
	{
		++$post;
		$times[] = $data['post_modified'];

		if ($post == $per || $data['post_id'] == $data['topic_last_post_id'])
		{
			$t_id = (int) $data['topic_id'];
			$f_id = (int) $data['forum_id'];
			$t_type = (int) $data['topic_type'];

			if ($seo)
			{
				if ($page == 0)
				{
					$phpbb_seo->prepare_iurl($data, 'topic', $t_type == POST_GLOBAL ? $phpbb_seo->seo_static['global_announce'] : $phpbb_seo->seo_url['forum'][$f_id]);
					$t_url = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $f_id . '&amp;t=' . $t_id);
				}
				else
				{
					$t_url = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $f_id . '&amp;t=' . $t_id . "&amp;start=" . ($page * $per));
				}

				$t_url = $phpbb_seo->drop_sid($t_url);
			}
			else
			{
				$no_f = (file_exists($phpbb_root_path . 'includes/hooks/hook_mini_seo.' . $phpEx)) ? 1 : 0;

				if ($page == 0)
				{
					$t_url = $path . "/viewtopic.$phpEx?" . (($no_f || $t_type == POST_GLOBAL) ? '' : "f=$f_id&amp;") . "t=$t_id";
				}
				else
				{
					$t_url = $path . "/viewtopic.$phpEx?" . (($no_f || $t_type == POST_GLOBAL) ? '' : "f=$f_id&amp;") . "t=$t_id&amp;start=" . ($page * $per);
				}
			}

			if ($urls && ($urls % ($limit * $index)) == 0)
			{
				$f_xml = $f_beg . $f_xml . $f_end;

				if (!($w = @fopen('sitemap/'.$index.'.xml', 'w')))
				{
					trigger_error(sprintf($user->lang['SEOMAP_CANT_WRITE'], $path . '/sitemap/'));
				}

				fwrite($w, $f_xml);
				fclose($w);

				$f_xml = '';
				++$index;
			}

			$time = gmdate('Y-m-d\TH:i:s+00:00', (int) max($times));
			$freq = $config['sitemap_seo_freq_' . $t_type . ''];
			$prior = $config['sitemap_seo_prior_' . $t_type . ''];

			add_to_record($t_url, $time, $freq, $prior);

			++$page;
			$times = array();
			$post = 0;

			if ($data['post_id'] == $data['topic_last_post_id'])
			{
				$page = 0;
			}
		}
	}
	$db->sql_freeresult($result);

	if ($f_xml)
	{
		$f_xml = $f_beg . $f_xml . $f_end;
	}
	else
	{
		trigger_error('SEOMAP_NO_DATA');
	}

	if ($index > 1)
	{
		if ($f_xml)
		{
			if (!($w = @fopen('sitemap/'.$index.'.xml', 'w')))
			{
				trigger_error(sprintf($user->lang['SEOMAP_CANT_WRITE'], $path . '/sitemap/'));
			}

			fwrite($w, $f_xml);
			fclose($w);
		}

		$f_xml = '<?xml version="1.0" encoding="UTF-8"?>'."\r\n";
		$f_xml .= '<?xml-stylesheet type="text/xsl" href="' . $path . '/styles/sitemap.xsl"?>'."\r\n";
		$f_xml .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'."\r\n";

		for ($i = 1; $i <= $index; $i++)
		{
			$f_xml .= '<sitemap>'."\r\n";
			$f_xml .= '<loc>' . $path . '/sitemap/' . $i . '.xml</loc>'."\r\n";
			$f_xml .= '<lastmod>' . gmdate('Y-m-d\TH:i:s+00:00', time()) . '</lastmod>'."\r\n";
			$f_xml .= '</sitemap>'."\r\n";
		}

		$f_xml .= '</sitemapindex>';
	}
}

/**
* Add URL record to sitemap
*/
function add_to_record($loc, $time, $freq, $prior)
{
	global $f_xml, $urls;

	$f_xml .= '<url>'."\r\n";
	$f_xml .= '<loc>'.$loc.'</loc>'."\r\n";

	if ($time)
	{
		$f_xml .= '<lastmod>'.$time.'</lastmod>'."\r\n";
	}
	if ($freq)
	{
		$f_xml .= '<changefreq>'.$freq.'</changefreq>'."\r\n";
	}
	if ($prior)
	{
		$f_xml .= '<priority>'.$prior.'</priority>'."\r\n";
	}
	$f_xml .= '</url>'."\r\n";

	++$urls;
}

?>